package eu.mhsl.minecraft.pixelblocks.listeners;

import eu.mhsl.minecraft.pixelblocks.PixelBlockItem;
import eu.mhsl.minecraft.pixelblocks.PixelBlocksPlugin;
import eu.mhsl.minecraft.pixelblocks.pixelblock.PixelBlock;
import eu.mhsl.minecraft.pixelblocks.pixelblock.PixelBlockWorld;
import eu.mhsl.minecraft.pixelblocks.utils.Direction;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.World;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.block.BlockPlaceEvent;
import org.bukkit.inventory.ItemStack;

import java.util.UUID;

public class PlacePixelBlockListener implements Listener {
    @EventHandler
    static void onBlockPlace(BlockPlaceEvent event) {
        ItemStack usedItem = event.getItemInHand();
        PixelBlockItem.BlockInfo info = PixelBlockItem.getBlockInfo(usedItem);
        if(info == null) return;

        World playerWorld = event.getPlayer().getWorld();
        if(PixelBlockWorld.isPixelWorld(playerWorld)) {
            event.getPlayer().sendMessage(Component.text("In Pixelblöcken kann kein Pixelblock platziert werden.", NamedTextColor.RED));
            event.setCancelled(true);
            return;
        }

        Location newBlockLocation = event.getBlock().getLocation();
        playerWorld.getBlockAt(newBlockLocation).setType(Material.AIR);

        Direction direction = Direction.vectorToDirection(event.getPlayer().getLocation().getDirection());

        PixelBlock pixelBlock;
        if(!info.hasOwner()) {
            pixelBlock = new PixelBlock(
                newBlockLocation,
                event.getPlayer().getUniqueId(),
                UUID.randomUUID(),
                direction
            );
        } else {
            UUID itemUUID = info.id();
            pixelBlock = PixelBlocksPlugin.pixelBlocks.stream()
                .filter(block -> block.blockUUID.equals(itemUUID))
                .findFirst()
                .orElseGet(() -> new PixelBlock(
                    newBlockLocation,
                    event.getPlayer().getUniqueId(),
                    itemUUID,
                    direction
                ));
        }

        try {
            pixelBlock.place(newBlockLocation, direction);
        } catch (IllegalArgumentException e) {
            event.setCancelled(true);
            event.getPlayer().sendMessage(Component.text(e.getMessage()));
        }
    }
}
