package eu.mhsl.minecraft.pixelblocks.pixelblock;

import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.block.data.BlockData;
import org.bukkit.entity.BlockDisplay;
import org.bukkit.entity.Entity;
import org.bukkit.entity.EntityType;
import org.bukkit.util.Transformation;
import org.jetbrains.annotations.NotNull;

import javax.annotation.Nullable;
import java.util.Objects;
import java.util.UUID;

public class Pixel {
    public final Location relativeLocation;
    public final BlockData blockData;
    private final double scale;

    private @Nullable UUID uuid;

    public Pixel(@NotNull Location relativeLocation, @NotNull BlockData blockData, double scale) {
        this.relativeLocation = new Location(
            relativeLocation.getWorld(),
            relativeLocation.x(),
            relativeLocation.y(),
            relativeLocation.z()
        );
        this.blockData = blockData;
        this.scale = scale;
    }

    public void place(@NotNull Location spawnBlockLocation) {
        World world = spawnBlockLocation.getWorld();
        double positionX = spawnBlockLocation.x() + (relativeLocation.x()*scale);
        double positionY = spawnBlockLocation.y() + (relativeLocation.y()*scale);
        double positionZ = spawnBlockLocation.z() + (relativeLocation.z()*scale);

        Location spawnLocation = new Location(world, positionX, positionY, positionZ);

        BlockDisplay entity = (BlockDisplay) world.spawnEntity(spawnLocation, EntityType.BLOCK_DISPLAY);
        this.uuid = entity.getUniqueId();

        entity.setBlock(blockData);

        Transformation transform = entity.getTransformation();
        transform.getScale().set(scale);
        entity.setTransformation(transform);
    }

    public void destroy() {
        Objects.requireNonNull(this.uuid);
        Entity pixelEntity = this.relativeLocation.getWorld().getEntity(this.uuid);
        if(pixelEntity != null) pixelEntity.remove();
    }
}
